@@ -1,22 +1,8 @@
-# Copyright 2014 Facebook, Inc.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 
-# You are hereby granted a non-exclusive, worldwide, royalty-free license to
-# use, copy, modify, and distribute this software in source code or binary
-# form for use in connection with the web services and APIs provided by
-# Facebook.
-
-# As with any software that integrates with the Facebook platform, your use
-# of this software is subject to the Facebook Developer Principles and
-# Policies [http://developers.facebook.com/policy/]. This copyright notice
-# shall be included in all copies or substantial portions of the software.
-
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# This source code is licensed under the license found in the
+# LICENSE file in the root directory of this source tree.
 
 from facebook_business.session import FacebookSession
 from facebook_business import apiconfig
@@ -36,9 +22,15 @@ from six.moves import http_client
 import os
 import json
 import six
-import collections
 import re
 
+try:
+  # Since python 3
+  from six.moves import collections_abc
+except ImportError:
+  # Won't work after python 3.8
+  import collections as collections_abc
+
 from facebook_business.adobjects.objectparser import ObjectParser
 from facebook_business.typechecker import TypeChecker
 
@@ -93,14 +85,14 @@ class FacebookResponse(object):
 
         json_body = self.json()
 
-        if isinstance(json_body, collections.Mapping) and 'error' in json_body:
+        if isinstance(json_body, collections_abc.Mapping) and 'error' in json_body:
             # Is a dictionary, has error in it
             return False
         elif bool(json_body):
             # Has body and no error
             if 'success' in json_body:
                 return json_body['success']
-            # API can retuen a success 200 when service unavailable occurs
+            # API can return a success 200 when service unavailable occurs
             return 'Service Unavailable' not in json_body
         elif self._http_status == http_client.NOT_MODIFIED:
             # ETAG Hit
@@ -281,7 +273,7 @@ class FacebookAdsApi(object):
 
         api_version = api_version or self._api_version
 
-        if api_version and not re.search('v[0-9]+\.[0-9]+', api_version):
+        if api_version and not re.search(r'v[0-9]+\.[0-9]+', api_version):
             raise FacebookBadObjectError(
                 'Please provide the API version in the following format: %s'
                 % self.API_VERSION,
@@ -667,6 +659,7 @@ class FacebookRequest:
                 api=self._api,
                 node_id=self._node_id,
                 endpoint=self._endpoint,
+                object_parser=self._response_parser,
             )
             cursor.load_next_page()
             return cursor
@@ -733,7 +726,7 @@ class Cursor(object):
         Args:
             source_object: An AbstractObject instance from which to inspect an
                 edge. This object should have an id.
-            target_objects_class: Objects traverersed over will be initialized
+            target_objects_class: Objects traversed over will be initialized
                 with this AbstractObject class.
             fields (optional): A list of fields of target_objects_class to
                 automatically read in.
@@ -840,8 +833,14 @@ class Cursor(object):
         response = response_obj.json()
         self._headers = response_obj.headers()
 
-        if 'paging' in response and 'next' in response['paging']:
-            self._path = response['paging']['next']
+        if (
+            'paging' in response and
+            'cursors' in response['paging'] and
+            'after' in response['paging']['cursors'] and
+            # 'after' will always exist even if no more pages are available
+            'next' in response['paging']
+        ):
+            self.params['after'] = response['paging']['cursors']['after']
         else:
             # Indicate if this was the last page
             self._finished_iteration = True
@@ -883,7 +882,7 @@ def _top_level_param_json_encode(params):
 
     for param, value in params.items():
         if (
-            isinstance(value, (collections.Mapping, collections.Sequence, bool))
+            isinstance(value, (collections_abc.Mapping, collections_abc.Sequence, bool))
             and not isinstance(value, six.string_types)
         ):
             params[param] = json.dumps(
